import CSI 1.0
import QtQuick

// Config LOADER

/*
Accessing local files
By default, you cannot use the XMLHttpRequest object to read files from your local file system. If you wish to use this feature to access local files, you can set the following environment variables to 1.

QML_XHR_ALLOW_FILE_READ
QML_XHR_ALLOW_FILE_WRITE
Warning: Use this feature only if you know that the application runs trusted QML and JavaScript code.
*/

Module {
  id: configLoader
  
  AppProperty { id: traktorPath; path: "app.traktor.settings.paths.root" }

  readonly property string _XMLHTTP_GET       : "GET"
  readonly property string _XMLHTTP_PUT       : "PUT"
  readonly property bool   _XMLHTTP_SYNCHRO   : false
  readonly property int    _XMLHTTP_DONE      : 4

  signal fileWritten( url fileUrl )
  onFileWritten: ( fileUrl )=>{ 
  }
  signal fileReaded( url fileUrl, string result )
  onFileReaded: ( fileUrl, result )=>{ 
  }

  Component.onCompleted: { 
    var configpath = configLoader.readConfigPath();
      
    // Read configuration
    var result = openFile(configpath)

    var prefsTemp = {};
    try {
      prefsTemp = JSON.parse(result);

    } catch (e) {
      prefs.log(e.message);

      // Save configuration (if not exit/ parse error)
      saveFile(configpath, JSON.stringify(prefs));
      result = openFile(configpath)
      prefsTemp = JSON.parse(result);
    }

    prefs.fontName = prefsTemp.fontName;
    prefs.fontMediumName = prefsTemp.fontMediumName;
    prefs.fontDeckHeaderNumber = prefsTemp.fontDeckHeaderNumber;
    prefs.fontDeckHeaderString = prefsTemp.fontDeckHeaderString;
    prefs.fontDeckFooterNumber = prefsTemp.fontDeckFooterNumber;
    prefs.fontDeckFooterString = prefsTemp.fontDeckFooterString;

    prefs.displayAlbumCover = prefsTemp.displayAlbumCover;
    prefs.displayHotCueBar = prefsTemp.displayHotCueBar;
    prefs.displayDeckIndicators = prefsTemp.displayDeckIndicators;
    prefs.displayPhaseMeter = prefsTemp.displayPhaseMeter;

    prefs.mixerFXSelector = prefsTemp.mixerFXSelector;
    prefs.prioritizeFXSelection = prefsTemp.prioritizeFXSelection;
    prefs.fineMasterTempoAdjust = prefsTemp.fineMasterTempoAdjust;
    prefs.fineDeckTempoAdjust = prefsTemp.fineDeckTempoAdjust;

    prefs.mixerFXNames = prefsTemp.mixerFXNames;

    prefs.spectrumWaveformColors = prefsTemp.spectrumWaveformColors;

    prefs.camelotKey = prefsTemp.camelotKey;
    prefs.phraseLength = prefsTemp.phraseLength;

    prefs.fullHeightBeatgridLines = prefsTemp.fullHeightBeatgridLines;
    prefs.fullHeightRedBeatgridLines = prefsTemp.fullHeightRedBeatgridLines;
    prefs.heightBeatgridLines = prefsTemp.heightBeatgridLines;
    prefs.heightRedBeatgridLines = prefsTemp.heightRedBeatgridLines;

    prefs.debug = prefsTemp.debug;

    prefs.log("Config debug:" + prefs.debug);

    prefs.log("       fontName:" + prefs.fontName);
    prefs.log("       fontMediumName:" + prefs.fontMediumName);
    prefs.log("       fontDeckHeaderNumber:" + prefs.fontDeckHeaderNumber);
    prefs.log("       fontDeckHeaderString:" + prefs.fontDeckHeaderString);
    prefs.log("       fontDeckFooterNumber:" + prefs.fontDeckFooterNumber);
    prefs.log("       fontDeckFooterString:" + prefs.fontDeckFooterString);

    prefs.log("       displayAlbumCover:" + prefs.displayAlbumCover);
    prefs.log("       displayHotCueBar:" + prefs.displayHotCueBar);
    prefs.log("       displayDeckIndicators:" + prefs.displayDeckIndicators);
    prefs.log("       displayPhaseMeter:" + prefs.displayPhaseMeter);

    prefs.log("       mixerFXSelector:" + prefs.mixerFXSelector);
    prefs.log("       prioritizeFXSelection:" + prefs.prioritizeFXSelection);
    prefs.log("       fineMasterTempoAdjust:" + prefs.fineMasterTempoAdjust);
    prefs.log("       fineDeckTempoAdjust:" + prefs.fineDeckTempoAdjust);

    prefs.log("       mixerFXNames:" + prefs.mixerFXNames);

    prefs.log("       spectrumWaveformColors:" + prefs.spectrumWaveformColors);

    prefs.log("       fullHeightBeatgridLines:" + prefs.fullHeightBeatgridLines);
    prefs.log("       fullHeightRedBeatgridLines:" + prefs.fullHeightRedBeatgridLines);
    prefs.log("       heightBeatgridLines:" + prefs.heightBeatgridLines);
    prefs.log("       heightRedBeatgridLines:" + prefs.heightRedBeatgridLines);

    prefs.log("       camelotKey:" + prefs.camelotKey);
    prefs.log("       phraseLength:" + prefs.phraseLength);
  }

  function readConfigPath() 
  {
    var configpath = "";
    if (prefs.osMode === 1) {
      configpath = traktorPath.value;
      configpath = "file:///" + configpath.replace(/\\/g,"/") + "/Settings.json";
    } else {
      configpath = traktorPath.value;
      configpath = "file:///Volumes/" + filePath.replace(/:/g, "/") + "Settings.json";
    }
    
    return configpath;
  }

  function openFile(fileUrl) {
      var request = new XMLHttpRequest();
      request.open(_XMLHTTP_GET, fileUrl, _XMLHTTP_SYNCHRO);
      request.send(null);
      return request.responseText;
  }

  function saveFile(fileUrl, fileContent) {
      var request = new XMLHttpRequest()
      request.open(_XMLHTTP_PUT, fileUrl, true)
      request.onreadystatechange = function() {
          if( request.readyState === 4 ) {
              fileWritten(fileUrl)
          }
      }
      request.send(fileContent)
  }
}
