import CSI 1.0
import QtQuick
import Qt5Compat.GraphicalEffects
import Traktor.Gui 1.0 as T

import '../../Defines'

Item {
  id: view

  Colors {id: colors}  

  property int deckId:          deckInfo.deckID
  property int streamId:        0 
  property int sampleWidth:     0  
  property var waveformColors:  colors.getDefaultWaveformColors(deckInfo.deckId)
  property var waveformPosition 

  readonly property var audioStreamKey: (primaryKey.value == 0) ? ["MixerDeckKey", view.deckId] 
                                                                : ["PrimaryKey", primaryKey.value, streamId]
  //--------------------------------------------------------------------------------------------------------------------

  AppProperty { id: primaryKey; path: "app.traktor.decks." + deckId + ".track.content.primary_key" }
  

  //--------------------------------------------------------------------------------------------------------------------

  T.Waveform {
    id: waveform
    anchors.fill:     parent
    deckId:           view.deckId
    waveformPosition: view.waveformPosition
    
    function calcLightColor(c, colorFactor) {
      return Qt.rgba( Math.min(1.0, colorFactor*c.r) ,  Math.min(1.0, colorFactor*c.g) , Math.min(1.0, colorFactor*c.b) , Math.min(1.0, colorFactor*c.a) );
    }

    function lightColor(c) {
      return calcLightColor(c, 1.6);
    }
    
    colorMatrix.high1: screen.width === 320 ? lightColor(waveformColors.high1) : waveformColors.high1
    colorMatrix.high2: screen.width === 320 ? lightColor(waveformColors.high2) : waveformColors.high2
    colorMatrix.mid1 : screen.width === 320 ? lightColor(waveformColors.mid1) : waveformColors.mid1
    colorMatrix.mid2 : screen.width === 320 ? lightColor(waveformColors.mid2) : waveformColors.mid2
    colorMatrix.low1 : screen.width === 320 ? lightColor(waveformColors.low1) : waveformColors.low1
    colorMatrix.low2 : screen.width === 320 ? lightColor(waveformColors.low2) : waveformColors.low2

    audioStreamKey:  (!view.visible) ? ["PrimaryKey", 0] : view.audioStreamKey
  }

  //--------------------------------------------------------------------------------------------------------------------

  WaveformColorize { 
    id: waveformColorize
    anchors.fill:  parent
    loop_start:    waveformCues.cueStart
    loop_length:   waveformCues.loop_length
    visible:       waveformCues.is_looping

    waveform:         waveform
    waveformPosition: view.waveformPosition
  }

}
